
TITLE	'CPM3 MODULE FOR the FPGA_DC + FDC DISK  CONTROLLER.'

;
;	12/17/1010	V1.0			John Monahan (monahan@vitasoft.org)
;	12/24/2010	V1.1			Added Timeout loop in Sector R/W.
;	02/19/2011	V1.2			Updated for new ZFDC commands
;	02/19/2011	V1.2			Updated for new ZFDC commands
;	10/01/2021	V1.2			Updated for new FPGA_DC Board

;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;   >>>>>>>>>>>>>>>>>>>>>>> THIS IS FOR A NON BANKED CPM3 SYSTEM <<<<<<<<<<<<<<<<<<<<<<<<<<<<
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

LF	EQU	0AH
CR	EQU	0DH
BELL	EQU	07H
CLEAR	EQU	1AH				;Propeller board Clear Screen
TAB	EQU	09H
ESC	EQU	1BH
EOL	EQU	1CH				;Propeller board Clear to end of line

	;  DEFINE LOGICAL VALUES:
TRUE:		EQU	-1
FALSE		EQU	NOT TRUE

	;  DETERMINE IF BANKED SELECT OR NOT
BANKED		EQU	FALSE				;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

;	PORTS  FOR Propeller Based Console IO Board

CIN		EQU	1H			;IN Data SD Systems Video Board
COUT		EQU	1H			;OUT Data SD Systems Video Board
CSTAT		EQU	0H			;Status Port SD Systems Video Board
	

;	PORTS FOR FOR Z80/WD2793 FDC on FPGA_DC Board to communicate with S100 BUS CPU

STATUS$BUS$PORT		EQU	80H		;CMD Communications port with Z80 on FPGA_DC Board
DATA$BUS$PORT		EQU	81H		;CMD data is sent via this port
RELEASE$ERROR$PORT	EQU	82H		;A pulse out to this port will reset the Sertor R/W Error flag on the FPGA Z80
REBOOT$PORT		EQU	83H		;Outputting anything to this port reboots the Z80 on the FPGA_DC Board
ABORT$CMD$PORT		EQU	84H		;Outputting 80H to port 84H will flag the FPGA to abort a currently running CMD


;----------------------------- FPGA_DC Board COMMANDS ----------------------------------                

						;Note these commands are a sub-set of the old ZFDC and ZFDC_II Board commands
						;Only a few are used here.

CMD$SEL$DRIVE		EQU	5H		;**** (Re)select an already initilized drive
CMD$SET$SIDE		EQU	8H		;**** This will set the active side for a floppy disk
CMD$SET$TRK$SEC		EQU	35H		;Set new current TRACK+SECTOR on current drive (new)
CMD$READ$SECTOR		EQU	10H		;**** Read data from the CURRENT sector (on current track,drive).
CMD$WRITE$SECTOR	EQU	11H		;**** Write data to the CURRENT sector (on current track,drive).
CMD$HANDSHAKE		EQU	21H		;**** Handshake command only sent during board initilization/testing 
CMD$SET$IDE$MODE	EQU	38H		;Set IDE drive hardware mode on FPG_DC Board (Internal or External)
						;Second Byte 0 = External IDE mode + Interrupt driven info (Default Mode). 
						;Second Byte 1 = External IDE mode + CMD driven info
						;Second Byte 2 = Local FPGA Board IDE mode + CMD driven info

CMD$FAST$COPY$AB	EQU	39H		;Special CMD to fast copy CF Card A:->B: (All within FPGA_DC Board software)

CMD$SET$HOME		EQU	0AH		;This will set floppy disk head request to Track 0 of CURRENT drive
CMD$GET$TRACK$ID	EQU	0FH		;Read the CURRENT TRACK ID
CMD$STEP$IN		EQU	0BH		;Step head in one track of CURRENT drive
CMD$STEP$OUT		EQU	0CH		;Step head out one track of CURRENT drive
CMD$SEEK$NV		EQU	0DH		;Seek to track with NO verify of CURRENT drive
CMD$SEEK$TRACK		EQU	0EH		;Seek to track to (IX+TRK) with the track verify bit set on CURRENT drive/format
CMD$TRACK$DUMP		EQU	15H		;Dump complete CURRENT track to S-100 system
						
INFO$READ$SECTOR	EQU	42H		;Send info, read sector (Actual Sec Read done externally)
INFO$WRITE$SECTOR	EQU	43H		;Send info, write sector
CMD$FORMAT$DISK		EQU	44H		;Format the disk in the of the CURRENT drive
CMD$FDC$COPY		EQU	45H		;Fast Copy Disk C:->D:

INFO$SEL$DRIVE		EQU	40H		;Send info, drive select
;INFO$SET$TRK$SEC	EQU	41H		;Send info, set track,sector
INFO$POST$MSG		EQU	41H		;Post a message on the USB Port Terminal
CMD$BOARD$CONFIG	EQU	52H		;Get FPGA_DC Board hardware configuration

CMD$GET$TIME		EQU	80H		;Get Time and Data from FPGA DC Board RTC
CMD$SET$TIME		EQU	81H		;Set Time and Data on FPGA DC Board RTC

						;Bits 7&6 contain IDE & FDC circuit flags 
						;Bits 1&0 contain IDE Mode (0,1,2) 

CMD$FAILED		EQU	0FFH		;CMD failed to exicute completely
CMD$COMPLETE		EQU	0F0H		;Flag returned by DC board to signal the previous CMD was completed OK
CMD$ABORT		EQU	0E0H		;CMD to abort the current process



;-------------------------------- CPM3 EXTERNALS ETC. --------------------------------------

	; DEFINE PUBLIC LABELS:
	PUBLIC	DPH2,DPH3,DPH4,DPH5	;FLOPPY DISK PARAMETER HEADERS


	; DEFINE EXTERNAL LABELS:
	EXTRN	@ADRV,@RDRV
	EXTRN	@DMA,@TRK,@SECT
	EXTRN	@CBNK
	EXTRN	@DBNK			;BANK FOR DMA OPERATION
	EXTRN	@ERMDE			;BDOS ERROR MODE
	EXTRN	?WBOOT			;WARM BOOT VECTOR
	EXTRN	?PMSG			;PRINT MESSAGE @<HL> UP TO 00, SAVES [BC] AND [DE]
	EXTRN	?PDERR			;PRINT BIOS DISK ERROR HEADER
	EXTRN	?CONIN,?CONO		;CONSOLE IN AND OUT
	EXTRN	?CONST			;CONSOLE STATUS
;	EXTRN	?BNKSL			;SELECT PROCESSOR MEMORY BANK (Not Used here)

;	EXTRN	?SMSG			;MY ROUTINE TO SPEAK A MESSAGE (Not Used here)

	; INCLUDE CP/M 3.0 DISK DEFINITION MACROS:
	MACLIB	CPM3

	; INCLUDE Z-80 MACRO LIBRARY:
	MACLIB	Z80
	
	; EXTENDED DISK PARAMETER HEADER FOR DRIVE 0: (C:)
	DW	WRITE$SECTOR		;FD SEC WRITE ROUTINE
	DW	READ$SECTOR		;FD SEC READ ROUTINE
	DW	FLOPPY$LOGIN$0		;FLOPPY DISK "C:" LOGIN PROCEDURE
	DW	FLOPPY$INIT$0		;FLOPPY DISK "C:" DRIVE INITIALIZATION ROUTINE
	DB	0			;RELATIVE DRIVE 0 ON THIS CONTROLLER (not used here)
	DB	0			;MEDIA TYPE KNOWN SSSD 8" 
					;HI BIT SET : DRIVE NEEDS RECALIBRATING

DPH2:	DPH	SD128$trans,SDSS128$dpb,,
					;Bytes 0-24 (above) are used by the above DPH (Pointed to by [DE] returned in the BIOS call)
	DW	128			;Byte 25 (here), is the value of 128 Bytes per sector count
	DB	'C'			;Byte 27 (here), is the value of the Drive Hardware Select CMD byte

	
	; EXTENDED DISK PARAMETER HEADER FOR DRIVE 1: (D:)
	DW	WRITE$SECTOR		;FD SEC WRITE ROUTINE
	DW	READ$SECTOR		;FD SEC READ ROUTINE
	DW	FLOPPY$LOGIN$1		;FLOPPY DISK "D:" LOGIN PROCEDURE
	DW	FLOPPY$INIT$1		;FLOPPY DISK DRIVE "D:" INITIALIZATION ROUTINE
	DB	1			;RELATIVE DRIVE 1 ON THIS CONTROLLER  (not used here)
	DB	0			;MEDIA TYPE KNOWN SSSD 8" 
					;HI BIT SET : DRIVE NEEDS RECALIBRATING

DPH3:	DPH	SD128$trans,SDSS128$dpb,,
					;Bytes 0-24 (above) are used by the above DPH (Pointed to by [DE] returned in the BIOS call)
	DW	128			;Byte 25 (here), is the value of 128 Bytes per sector count
	DB	'D'			;Byte 27 (here), is the value of the Drive Hardware Select CMD byte


	; EXTENDED DISK PARAMETER HEADER FOR DRIVE 2: (E:)
	DW	WRITE$SECTOR		;FD SEC WRITE ROUTINE
	DW	READ$SECTOR		;FD SEC READ ROUTINE
	DW	FLOPPY$LOGIN$2		;FLOPPY DISK "E:" LOGIN PROCEDURE
	DW	FLOPPY$INIT$2		;FLOPPY DISK "E:" DRIVE INITIALIZATION ROUTINE
	DB	2			;RELATIVE DRIVE 2 ON THIS CONTROLLER  (not used here)
	DB	0			;MEDIA TYPE KNOWN 8" (IBM System 34 Format)
					;HI BIT SET : DRIVE NEEDS RECALIBRATING

DPH4:	DPH	DD256$trans,DDDS256$dpb,,
					;Bytes 0-24 (above) are used by the above DPH (Pointed to by [DE] returned in the BIOS call)
	DW	256			;Byte 25 (here), is the value of 256 Bytes per sector count
	DB	'E'			;Byte 27 (here), is the value of the Drive Hardware Select CMD byte


	; EXTENDED DISK PARAMETER HEADER FOR DRIVE 2: (F:)
	DW	WRITE$SECTOR		;FD SEC WRITE ROUTINE
	DW	READ$SECTOR		;FD SEC READ ROUTINE
	DW	FLOPPY$LOGIN$3		;FLOPPY DISK "F:" LOGIN PROCEDURE
	DW	FLOPPY$INIT$3		;FLOPPY DISK "F:" DRIVE INITIALIZATION ROUTINE
	DB	3			;RELATIVE DRIVE 3 ON THIS CONTROLLER  (not used here)
	DB	0			;MEDIA TYPE KNOWN 3.5" (IBM PC 1.44MG Format)
					;HI BIT SET : DRIVE NEEDS RECALIBRATING

DPH5:	DPH	DD512$trans,DDT144$dpb,,
					;Bytes 0-24 (above) are used by the above DPH (Pointed to by [DE] returned in the BIOS call)
	DW	512			;Byte 25 (here), is the value of 512 Bytes per sector count
	DB	'F'			;Byte 27 (here), is the value of the Drive Hardware Select CMD byte



	; CPM3 XDPH offset definitions:-
					;Offset constants for the above XDPH's 
BYTE$COUNT	equ	25		;# bytes/sector for a disk. The CPM sent BIOS [DE] offset
DRIVE$SELECT	equ	27		;Drive Hardware Select.  The CPM sent BIOS [DE] offset
	

	; 128 byte sectors, SD, SS disk parameter block (IBM 3740):
	
SDSS128$dpb:	dpb	128,26,77,1024,64,2	;8" SDSS Disks
DDDS256$dpb:	dpb	256,26,77,2048,64,2	;8" DSDD Disks
DDT144$dpb:	dpb	512,36,80,2048,128,1	;5" Disks (18 = 12H) (36=18X2 = 24H), because 1 Track, but both sides)

SD128$trans:	skew	26,6,0		;8" sector skew (Note first sector is 0)
DD256$trans:	skew	26,6,0		;8" sector skew (Note first sector is 0)
DD512$trans:	skew	36,4,0		;3.5" sector skew (36 = 18X2) (Note first sector is 0)

@CURRENT$DRIVE	DB	0FFH		;Current Selecte Floppy Disk Drive



	;;;;; INIT:			;Initilization routines for each floppy drive

FLOPPY$INIT$0:
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE			
	RET				;All drives will be initilized after reset on the FPGA_DC Board

FLOPPY$INIT$1:				;Next the second floppy drive, Just return
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE			
	RET

FLOPPY$INIT$2:				;Then the third floppy drive, Just return
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE			
	RET
	
FLOPPY$INIT$3:				;Then the third floppy drive, Just return
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE			
	RET



;-----------------LOGIN OF FLOPPY DISK -----------------------------------------
;
;	on entry:	[DE] = pointer to XDPH of drive to be logged-in
;
;	on exit:	XDPH address on the stack at entry has been
;			replaced with a null address if the drive has
;			been determined to be an invalid one
;
;			Note this routine is called only the FIRST time a floppy disk is accessed
;			It is also called after a ^C character is entered from keyboard
;
FLOPPY$LOGIN$0:				;Select Floppy Drive C:
FLOPPY$LOGIN$1:	
FLOPPY$LOGIN$2:		
FLOPPY$LOGIN$3:		
	PUSH	D			;Save it just in case CPM uses it later
	PUSHIX				;Save [IX]
	PUSH	D			;[DE]->[IX]
	POPIX				;Get XDPH address for current requested drive to [IX]

	MVI	A,CMD$SEL$DRIVE		;05H,  Select Drive CMD, drive in [C]
	CALL	DATA$TO$FPGA
	LDX	A,DRIVE$SELECT		;[A] <- (IX + DRIVE$SELECT). Floppy Drive #, (FPGA_DC Board expects a C,D,E or F)
	MOV	D,A			;Store selected drive in [D]
	
	CALL	DATA$TO$FPGA		
	CALL	GET$FPGA$TO$S100	;Wait for returned ACK signal 
	CMP	D			;Was it the correct drive ack
	JNZ	LOGIN$ERROR

	STA	@CURRENT$DRIVE
	
	MVI	A,CMD$SET$HOME		;This will set floppy disk head request to Track 0 of CURRENT drive
	CALL	DATA$TO$FPGA

	CALL	GET$FPGA$TO$S100	;Get CMD$CPMPLETE/ERROR FLAG
	CPI	CMD$COMPLETE		;0F0H, Flag returned by DC board to signal the previous CMD was completed OK
	JNZ	LOGIN$ERROR	

	XRA	A
	POPIX				;Return with origional IX
	POP	D			;And origional DE
	RET				;Return Z as no error
	

LOGIN$ERROR:
	LXI	H,FLOPPY$LOGIN$ERR	;'Floppy Disk Drive Login Error.',0
	CALL	?PMSG			;Write error message
	XRA	A
	DCR	A
	POPIX				;Return with origional IX
	POP	D			;[DE] has calling routine's return address
	LXI	H,0			;This will be invalid drive from now on in the CPM BDOS
	RET




;---------------FLOPPY SECTOR READ ROUTINE ------------------------------------------------
;
;		ROUTINE READS 1 SECTOR TO THE DISK:
;
;	on entry:	[de] = XDPH address for current drive
;
;	on exit:	[a] =   0 --> successful read operation
;				1 --> unsuccessful read operation
;
; Assumes valid track in (@TRK) & (@SECT), Address in (@DMA). 
; If the disk is double sided then the ZFDC board assumes 1 to NSCTRS+1 on Side A 
; and NSCTRS+1 to (NSCTRS X 2)+1 on Side B
;
; Note: The XDPH table is used to get the current drive and sector byte count.  
; These are  IX+DRIVE$SELECT and IX+BYTE$COUNT from the above FLOPPY$LOGIN routine.
;--------------------------------------------------------------------------------------------

READ$SECTOR:
	PUSH	D			;Save it just in case CPM uses it later
	PUSHIX				;Save [IX]
	PUSH	D			;[DE]->[IX]
	POPIX				;get XDPH address for current requested drive to [ix]
	
	LDX	D,DRIVE$SELECT		;[D] <- (IX + DRIVE$SELECT). Floppy Drive #
	LDA	@CURRENT$DRIVE		;Is drive already selected
	CMP	D
	JZ	RD$CURRENT$DRIVE
	
	MVI	A,CMD$SEL$DRIVE		;<<< Set Drive.  Will just return if current drive
	CALL	DATA$TO$FPGA		;Note no restore/home CMD is required
	MOV	A,D			;Store selected drive in [D]
	CALL	DATA$TO$FPGA		

	CALL	GET$FPGA$TO$S100	;Wait for returned ACK signal 
	CMP	D			;Was it the correct drive ack
	JNZ	DRIVE$RD$SEL$ERROR
	STA	@CURRENT$DRIVE		;Store for next time
	
RD$$CURRENT$DRIVE:
	MVI	A,CMD$READ$SECTOR	;10H,  Send CMD_READ_SECTOR command 
	CALL	DATA$TO$FPGA

	LHLD  	@DMA			;Send DMA address (Just for info display)
	MOV	A,H
	CALL	DATA$TO$FPGA
	MOV	A,L
	CALL	DATA$TO$FPGA

	LDA	@TRK			;Send Track
	CALL	DATA$TO$FPGA
	
	LDA	@SECT			;Send Sector
	INR	A			;Sectors are numbered 1,2,3
	CALL	DATA$TO$FPGA

	LHLD    @DMA			;Get DMA address
	LDX	E,BYTE$COUNT		;[E] <- (IX + BYTE$COUNT).   Sector size (128, 256 or 512)
	LDX	D,BYTE$COUNT+1		;[D] <- (IX + BYTE$COUNT+1). Sector size
	

FDC$SEC$RD$LOOP:			;<<<< LOOP CF/SD Card Sector READ >>>>
	CALL	GET$FPGA$TO$S100	;Get data in [A] back to S100 
	JC	SECTOR$READ$ERROR	;If error break out of loop
	MOV	M,A
	INX	H
	DCX	D			;Decrease DE
	MOV	A,E
	ORA	D
	JNZ	FDC$SEC$RD$LOOP		;Get all 128, 256 or 512 bytes

	CALL	GET$FPGA$TO$S100	;Get CMD$CPMPLETE/ERROR FLAG
	CPI	CMD$COMPLETE		;0F0H, Flag returned by DC board to signal the previous CMD was completed OK
	JNZ	SECTOR$READ$ERROR	
	XRA	A
	POPIX				;Return with origional IX
	POP	D			;and DE
	RET				;Ret Z if all OK


SECTOR$READ$ERROR:
	LXI	H,FLOPPY$READ$ERR	;"Sector Read Error. Track"
	CALL	?PMSG
	LDA	@TRK
	CALL	HHEXOUT
	LXI	H,SEC$NUMBER		;"Sector"
	CALL	?PMSG
	LDA	@SECT
	INR	A			;Remember sectors 1,2,3
	CALL	HHEXOUT

	OUT	REBOOT$PORT		
	LXI	H,BOARD$RESET		;Will reboot the FPGA_DC board
	CALL	?PMSG
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE		;Force a new drive login		

	POPIX				;Return with origional IX
	POP	D			;and DE
	XRA	A
	DCR	A			;Set to NZ & 1 for read error
	RET		

DRIVE$RD$SEL$ERROR:
	LXI	H,FLOPPYRD$$SEL$ERR	;"Drive Select Error for Sec Read"
	CALL	?PMSG
	POPIX				;Return with origional IX
	POP	D			;and DE
	XRA	A
	DCR	A			;Set to NZ & 1 for read error
	RET		

;---------------FLOPPY SECTOR WRITE ROUTINE -------------------------------------------
;
;		ROUTINE WRITES 1 SECTOR TO THE DISK:
;
;	on entry:	[de] = XDPH address for current drive
;
;	on exit:	[a] =   0 --> successful write operation
;				1 --> unsuccessful write operation
;                               2 --> Disk is write protected
;
; Assumes valid track in (@TRK) & (@SECT), Address in (@DMA) from CPM 
; If the disk is double sided then the FPGA_DC board assumes 1 to NSCTRS+1 on Side A 
; and NSCTRS+1 to (NSCTRS X 2)+1 on Side B
;
; Note: The XDPH table is used to get the current drive and sector byte count.  
; These are  IX+DRIVE$SELECT and IX+BYTE$COUNT from the above FLOPPY$LOGIN routine.
;----------------------------------------------------------------------------------------

WRITE$SECTOR:
	PUSH	D			;Save it just in case CPM uses it later
	PUSHIX				;Save [IX]
	PUSH	D			;[DE]->[IX]
	POPIX				;get XDPH address for current requested drive to [ix]
	
	LDX	D,DRIVE$SELECT		;[D] <- (IX + DRIVE$SELECT). Floppy Drive #
	LDA	@CURRENT$DRIVE		;Is drive already selected
	CMP	D
	JZ	WR$CURRENT$DRIVE
	
	MVI	A,CMD$SEL$DRIVE		;<<< Set Drive.  Will just return if current drive
	CALL	DATA$TO$FPGA		;Note no restore/home CMD is required
	MOV	A,D			;Store selected drive in [D]
	CALL	DATA$TO$FPGA		

	CALL	GET$FPGA$TO$S100	;Wait for returned ACK signal 
	CMP	D			;Was it the correct drive ack
	JNZ	DRIVE$RD$SEL$ERROR
	STA	@CURRENT$DRIVE		;Store for next time
	
WR$$CURRENT$DRIVE:
	MVI	A,CMD$WRITE$SECTOR	;10H,  Send CMD_READ_SECTOR command 
	CALL	DATA$TO$FPGA

	LHLD  	@DMA			;SEnd DMA address (Just for info display)
	MOV	A,H
	CALL	DATA$TO$FPGA
	MOV	A,L
	CALL	DATA$TO$FPGA

	LDA	@TRK			;Send Track
	CALL	DATA$TO$FPGA
	
	LDA	@SECT			;Send Sector
	INR	A			;Sectors are numbered 1,2,3
	CALL	DATA$TO$FPGA

	LHLD    @DMA			;Get DMA address
	LDX	E,BYTE$COUNT		;[E] <- (IX + BYTE$COUNT).   Sector size (128, 256 or 512)
	LDX	D,BYTE$COUNT+1		;[D] <- (IX + BYTE$COUNT+1). Sector size
	
FDC$SEC$WR$LOOP:			;<<<< LOOP Sector WRITE >>>>
	MOV	A,M
	CALL	DATA$TO$FPGA		;Send data in [A] to FPGA 
	INX	H
	DCX	D			;Decrease DE
	MOV	A,E
	ORA	D
	JNZ	FDC$SEC$WR$LOOP		;Get all 128 or 512 bytes

	CALL	GET$FPGA$TO$S100	;Get CMD$CPMPLETE/ERROR FLAG
	CPI	CMD$COMPLETE		;0F0H, Flag returned by DC board to signal the previous CMD was completed OK
	JNZ	SECTOR$WRITE$ERROR	
	XRA	A
	POPIX				;Return with origional IX
	POP	D			;and DE
	RET				;Ret Z if all OK


SECTOR$WRITE$ERROR:
	LXI	H,FLOPPY$WRITE$ERR	;"Sector Write Error"
	CALL	?PMSG
	LDA	@TRK
	CALL	HHEXOUT
	LXI	H,SEC$NUMBER		;"Sector"
	CALL	?PMSG
	LDA	@SECT
	INR	A			;Remember sectors 1,2,3
	CALL	HHEXOUT

	OUT	REBOOT$PORT		
	LXI	H,BOARD$RESET		;Will reboot the FPGA_DC board
	CALL	?PMSG
	MVI	A,0FFH	
	STA	@CURRENT$DRIVE		;Force a new drive login		

	LXI	H,FLOPPY$WRITE$ERR	;"Sector Write Error"
	CALL	?PMSG
	POPIX				;Return with origional IX
	POP	D			;and DE
	XRA	A
	DCR	A			;Set to NZ & 1 for read error
	RET		

DRIVE$WR$SEL$ERROR:
	LXI	H,FLOPPY$WR$SEL$ERR	;"Drive Select Error for Sec Write"
	CALL	?PMSG
	POPIX				;Return with origional IX
	POP	D			;and DE
	XRA	A
	DCR	A			;Set to NZ & 1 for read error
	RET		

;-------------------------------------COMMUNICATION WITH FPGA_DC BOARD -----------------------------------------------

DATA$TO$FPGA:				;Send CMD/data to FPGA_DC Z80
	PUSH	PSW
DATA$TO$FPGA1:
	IN	STATUS$BUS$PORT	
;	CPI	0FFH			;0FFH if we have the Dual IDE/CF Card 
;	JZ	NO$FPGA$DC
	ANI	80H
	JRNZ	DATA$TO$FPGA1		;Wait until bit 7 is 0 (ie. any previous data has been read by FPGA_DC board Z80)
	POP	PSW
	OUT	DATA$BUS$PORT		;This will cause the FPGA_DC board to set the S100$RTS bit
DATA$TO$FPGA2:
	IN	STATUS$BUS$PORT	
	ANI	80H
	JRNZ	DATA$TO$FPGA2		;Wait until bit 7 is 0 
	RET				;Ret NZ if error
NO$FPGA$DC
	POP	PSW
	RET
	

GET$FPGA$TO$S100:			;Get data from FPGA
	IN	STATUS$BUS$PORT		;Get S100 Status Port - Bit 0, 1 = FPGA data present, 0, no data present
	BIT	2,A			;Was theh error bit flaged
	JNZ	GOT$ERROR$FLAG
	ANI	01H			;01H, Bit 0
	JZ	GET$FPGA$TO$S100	;wait for DC Board to present data 
	IN	DATA$BUS$PORT		;Return Data in [A], FPGA "hardware" will lower bit 7 of S100 Bus port 80H
	RET

GOT$ERROR$FLAG:
	SCF
	RET				;Return with carry flag set if an error was detected on teh FPGA_DC Board

; -------------- For Debugging purposes ------------------------------------
;
; Print an 8 bit number, located in [A]
HEXOUT:	PUSH	PSW			;No registers altered
	PUSH	B
	PUSH	PSW
	RRC
	RRC
	RRC
	RRC
	CALL	ZCONV
	POP	PSW
	CALL	ZCONV
	POP	B
	POP	PSW
	RET


HHEXOUT:PUSH	PSW			;No registers altered
	PUSH	B
	CALL	HEXOUT
	MVI	C,'H'	
	CALL	ZCO
	POP	B
	POP	PSW
	RET
		

ZCONV:	ANI	0FH			;HEX to ASCII and print it
	ADI	90H
	DAA
	ACI	40H
	DAA
	MOV	C,A
	CALL	ZCO
	RET


ZCI:	IN	0H			;Direct console input (used only for debugging)
	ANI	02H
	JZ	ZCI
	IN	1H			;return with character in A
	RET
;		
ZCO:	IN	0H			;console output (arrive with character in C)
	ANI	04H			;Note character is in C and A on return.
	JZ	ZCO
	MOV	A,C
	OUT	1H
	RET
	
;--------------------------------------------------------------------------------
;
FLOPPY$LOGIN$ERR	DB	CR,LF,BELL,'Floppy Disk Drive Login Error.',0
FLOPPY$WRITE$ERR	DB	CR,LF,BELL,'FPGA_DC Board Sector Write Error. Track ',0 
FLOPPY$READ$ERR		DB	CR,LF,BELL,'FPGA Board Sector Read Error. Track ',0 
SEC$NUMBER		DB	' Sector ',0 
BOARD$RESET		DB	CR,LF,'Will reset the FPGA_DC Board.',0

FLOPPY$RD$$SEL$ERR	DB	CR,LF,BELL,'Drive Select Error for Sector Read.',0
FLOPPY$WR$SEL$ERR	DB	CR,LF,BELL,'Drive Select Error for Sector Write.',0
;END