
TITLE	'CPM3 MODULE FOR the FPGA_DC Controller Board.'

; Assume a NON-BANKED version of CPM3
;
;	12/28/2009	Corrected high/low byte R/W of data to sectors
;	03/15/2011	Single reset pulse to IDE board
;	03/16/2018	Check CF card is ready after RESET before issuing CMD
;	03/26/2021	Converted code from Dual IDE/CF card to work with the FPGA_DC Controller Board
;
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
;   >>>>>>>>>>>>>>>>>>>>>>> THIS IS FOR A NON BANKED CPM3 SYSTEM <<<<<<<<<<<<<<<<<<<<<<<<<<<<
;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

;	Note the CPM BIOS Stack space is very small. If you need more, setup a locak Stack

	;  DEFINE LOGICAL VALUES:
TRUE:		EQU	-1
FALSE		EQU	NOT TRUE

	;  DETERMINE IF BANKED SELECT OR NOT
BANKED		EQU	FALSE				;<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


BELL	EQU	07H
CR	EQU	0DH
LF	EQU	0AH

;	PORTS  FOR FOR Propeller Console IO Board (For debugging only)
CIN		EQU	1H		;IN Data SD Systems Video Board
COUT		EQU	1H		;OUT Data SD Systems Video Board
CSTAT		EQU	0H		;Status Port SD Systems Video Board

STATUS$BUS$PORT EQU	80H		;CMD Communications port with Z80 on FPGA_DC Board
DATA$BUS$PORT	EQU	81H		;CMD data is sent via this port

;Ports for 8255 chip. Change these to specify where the 8255 is addressed,
;and which of the 8255's ports are connected to which IDE signals.
;The first three control which 8255 ports have the control signals,
;upper and lower data bytes.  The last one is for mode setting for the
;8255 to configure its ports, which must correspond to the way that
;the first three lines define which ports are connected.

IDEportA	EQU	030H		;lower 8 bits of IDE interface
IDEportB	EQU	031H		;upper 8 bits of IDE interface
IDEportC	EQU	032H		;control lines for IDE interface
IDEportCtrl	EQU	033H		;8255 configuration port
IDEDrive        EQU     034H            ;Bit 0 = 0 for Drive A: = 1 for B:

READcfg8255	EQU	10010010b	;Set 8255 IDEportC out, IDEportA/B input
WRITEcfg8255	EQU	10000000b	;Set all three 8255 ports output

;IDE control lines for use with IDEportC.  Change these 8
;constants to reflect where each signal of the 8255 each of the
;IDE control signals is connected.  All the control signals must
;be on the same port, but these 8 lines let you connect them to
;whichever pins on that port.

IDEa0line	EQU	01H		;direct from 8255 to IDE interface
IDEa1line	EQU	02H		;direct from 8255 to IDE interface
IDEa2line	EQU	04H		;direct from 8255 to IDE interface
IDEcs0line	EQU	08H		;inverter between 8255 and IDE interface
IDEcs1line	EQU	10H		;inverter between 8255 and IDE interface
IDEwrline	EQU	20H		;inverter between 8255 and IDE interface
IDErdline	EQU	40H		;inverter between 8255 and IDE interface
IDErstline	EQU	80H		;inverter between 8255 and IDE interface
;
;Symbolic constants for the IDE Drive registers, which makes the
;code more readable than always specifying the address pins

REGdata		EQU	IDEcs0line
REGerr		EQU	IDEcs0line + IDEa0line
REGseccnt	EQU	IDEcs0line + IDEa1line
REGsector	EQU	IDEcs0line + IDEa1line + IDEa0line
REGcylinderLSB	EQU	IDEcs0line + IDEa2line
REGcylinderMSB	EQU	IDEcs0line + IDEa2line + IDEa0line
REGshd		EQU	IDEcs0line + IDEa2line + IDEa1line		;(0EH)
REGcommand	EQU	IDEcs0line + IDEa2line + IDEa1line + IDEa0line	;(0FH)
REGstatus	EQU	IDEcs0line + IDEa2line + IDEa1line + IDEa0line
REGcontrol	EQU	IDEcs1line + IDEa2line + IDEa1line
REGastatus	EQU	IDEcs1line + IDEa2line + IDEa1line + IDEa0line

;IDE Command Constants.  These should never change.

COMMANDrecal	EQU	10H
COMMANDread	EQU	20H
COMMANDwrite	EQU	30H
COMMANDinit	EQU	91H
COMMANDid	EQU	0ECH
COMMANDspindown	EQU	0E0H
COMMANDspinup	EQU	0E1H
					

; IDE Status Register:
;  bit 7: Busy	1=busy, 0=not busy
;  bit 6: Ready 1=ready for command, 0=not ready yet
;  bit 5: DF	1=fault occured insIDE drive
;  bit 4: DSC	1=seek complete
;  bit 3: DRQ	1=data request ready, 0=not ready to xfer yet
;  bit 2: CORR	1=correctable error occured
;  bit 1: IDX	vendor specific
;  bit 0: ERR	1=error occured

;----------------------------- FPGA_DC Board COMMANDS ----------------------------------                

						;Note these commands are a sub-set of the old ZFDC and ZFDC_II Board commands
						;Only a few are used here.

CMD$SEL$DRIVE		EQU	5H		;**** (Re)select an already initilized drive
CMD$SET$SIDE		EQU	8H		;**** This will set the active side for a floppy disk
CMD$SET$TRK$SEC		EQU	35H		;Set new current TRACK+SECTOR on current drive (new)
CMD$READ$SECTOR		EQU	10H		;**** Read data from the CURRENT sector (on current track,drive).
CMD$WRITE$SECTOR	EQU	11H		;**** Write data to the CURRENT sector (on current track,drive).
CMD$HANDSHAKE		EQU	21H		;**** Handshake command only sent during board initilization/testing 
CMD$SET$IDE$MODE	EQU	38H		;Set IDE drive hardware mode on FPG_DC Board (Internal or External)
						;Second Byte 0 = External IDE mode + Interrupt driven info (Default Mode). 
						;Second Byte 1 = External IDE mode + CMD driven info
						;Second Byte 2 = Local FPGA Board IDE mode + CMD driven info

CMD$FAST$COPY$AB	EQU	39H		;Special CMD to fast copy CF Card A:->B: (All within FPGA_DC Board software)

CMD$SET$HOME		EQU	0AH		;This will set floppy disk head request to Track 0 of CURRENT drive
CMD$GET$TRACK$ID	EQU	0FH		;Read the CURRENT TRACK ID
CMD$STEP$IN		EQU	0BH		;Step head in one track of CURRENT drive
CMD$STEP$OUT		EQU	0CH		;Step head out one track of CURRENT drive
CMD$SEEK$NV		EQU	0DH		;Seek to track with NO verify of CURRENT drive
CMD$SEEK$TRACK		EQU	0EH		;Seek to track to (IY+DRIVE_TRACK) with the track verify bit set on CURRENT drive/format
CMD$TRACK$DUMP		EQU	15H		;Dump complete CURRENT track to S-100 system
						
INFO$READ$SECTOR	EQU	42H		;Send info, read sector (Actual Sec Read done externally)
INFO$WRITE$SECTOR	EQU	43H		;Send info, write sector
CMD$FORMAT$DISK		EQU	44H		;Format the disk in the of the CURRENT drive
CMD$FDC$COPY		EQU	45H		;Fast Copy Disk C:->D:

;INFO$SEL$DRIVE		EQU	40H		;Send info, drive select
;INFO$SET$TRK$SEC	EQU	41H		;Send info, set track,sector
INFO$POST$MSG		EQU	41H		;Post a message on the USB Port Terminal
CMD$BOARD$CONFIG	EQU	52H		;Get FPGA_DC Board hardware configuration

CMD$GET$TIME		EQU	80H		;Get Time and Data from FPGA DC Board RTC
CMD$SET$TIME		EQU	81H		;Set Time and Data on FPGA DC Board RTC

						;Bits 7&6 contain IDE & FDC circuit flags 
						;Bits 1&0 contain IDE Mode (0,1,2) 

CMD$FAILED		EQU	0FFH		;CMD failed to exicute completely
CMD$COMPLETE		EQU	0F0H		;Flag returned by DC board to signal the previous CMD was completed OK
CMD$ABORT		EQU	0E0H		;CMD to abort the current process


EXT$IDE$MODE		EQU	00H		;MODE 0, FPGA IDE mode set to external IDE ports etc, 
LOCAL$IDE$MODE		EQU	01H		;MODE 1, FPGA Local IDE CMD mode   [CMD driven Hex display/info]
EXT$IDE$INFO$MODE	EQU	02H		;MODE 2, FPGA External IDE         [CMD driven Hex display/info]



;-------------------------------- CPM3 EXTERNALS ETC. --------------------------------------

	; DEFINE PUBLIC LABELS:
	PUBLIC	DPH0,DPH1		;DISK PARAMETER HEADERS
	
	
	
	; DEFINE EXTERNAL LABELS:
	EXTRN	@ADRV,@RDRV
	EXTRN	@DMA,@TRK,@SECT
	EXTRN	@CBNK
	EXTRN	@DBNK			;BANK FOR DMA OPERATION
	EXTRN	@ERMDE			;BDOS ERROR MODE
	EXTRN	?WBOOT			;WARM BOOT VECTOR
	EXTRN	?PMSG			;PRINT MESSAGE @<HL> UP TO 00, SAVES
					; [BC] AND [DE]
	EXTRN	?PDERR			;PRINT BIOS DISK ERROR HEADER
	EXTRN	?CONIN,?CONO		;CONSOLE IN AND OUT
	EXTRN	?CONST			;CONSOLE STATUS
	IF	BANKED
	EXTRN	?BNKSL			;SELECT PROCESSOR MEMORY BANK
	ENDIF

	; INCLUDE CP/M 3.0 DISK DEFINITION MACROS:
	MACLIB	CPM3

	; INCLUDE Z-80 MACRO LIBRARY:
	MACLIB	Z80

	IF BANKED
	DSEG				;PUT IN OP SYS BANK IF BANKING
	ENDIF

	; EXTENDED DISK PARAMETER HEADER FOR DRIVE 0:
	DW	HDWRT			;HARD DISK WRITE ROUTINE
	DW	HDRD			;HARD DISK READ ROUTINE
	DW	HDLOGIN$0		;HARD DISK LOGIN PROCEDURE
	DW	HDINIT$0		;HARD DISK DRIVE INITIALIZATION ROUTINE
	DB	0			;RELATIVE DRIVE 0 ON THIS CONTROLLER
	DB	0			;MEDIA TYPE:
					;  HI BIT SET : DRIVE NEEDS RECALIBRATING

DPH0:	DPH	0,IDEHD$DPB,0,	

        ; EXTENDED DISK PARAMETER HEADER FOR DRIVE 1:
        DW      HDWRT			;HARD DISK WRITE ROUTINE
        DW      HDRD			;HARD DISK READ ROUTINE
        DW      HDLOGIN$1		;HARD DISK LOGIN PROCEDURE
        DW      HDINIT$1		;HARD DISK DRIVE INITIALIZATION ROUTINE
        DB      0			;RELATIVE DRIVE 0 ON THIS CONTROLLER
        DB      0			;MEDIA TYPE:
					;  HI BIT SET : DRIVE NEEDS RECALIBRATING
DPH1:   DPH     0,IDEHD$DPB,0,


	; MAKE SURE DPB'S ARE IN COMMON MEMORY:
	CSEG

	; 512 BYTE SECTORS ON HARD DISK (512X61X256) = 7,995,392 Bytes)
	; with 16 heads this is less than 16 drive cylinders!
	
IDEHD$DPB:
;	DPB	512,32,512,2048,1024,2,8000H	
;	DPB	256,32,1001,2048,1024,2,8000H	
;	DPB	512,60,256,2048,1024,1,8000H
;	DPB	512,62,252,2048,1024,1,8000H
	DPB	512,61,256,2048,1024,1,8000H

@BOARD$CONFIG	DB	0H		;IDE, FDC Board configuration bits


	IF BANKED
	DSEG				;CAN SET BACK TO BANKED SEGMENT IF BANKING
	ENDIF



;;;;; HDINIT:
HDINIT$0:
	IN	STATUS$BUS$PORT		;If FPGA_DC is active this will be X000000X
	CPI	0FFH
	JNZ	GOT$RESPONSE
	LXI	H,NO$FPGA$BOARD$MSG	;'The FPGA_DC Board was not detected, will abort'
	CALL	PRINT$STRING
	RET

GOT$RESPONSE:
	MVI	A,CMD$SET$IDE$MODE	;38H set IDE to Mode 0
	CALL	DATA$TO$FPGA
;	MVI	A,LOCAL$IDE$MODE	;1H, Set to the FPGA_DC Local IDE mode + CMDs
;	MVI	A,EXT$IDE$INFO$MODE	;2H  Select External IDE FPGA circuit + CMD Info
	MVI	A,EXT$IDE$MODE		;0h  FPGA IDE mode set to external IDE ports etc 
	CALL	DATA$TO$FPGA
	LXI	H,MODE$2$$MSG		;'The FPGA_DC Board set to mode 0
	CALL	PRINT$STRING

	MVI	A,CMD$BOARD$CONFIG	;Get the current FPGA_DC Board configuration
	CALL	DATA$TO$FPGA
	
	CALL	GET$FPGA$TO$S100	;Wait for returned signal 

	STA	@BOARD$CONFIG

	ANI	11000000B
	CPI	11000000B
	JZ	BOTH$CIRCUITS
	CPI	10000000B
	JZ	IDE$CIRCUIT
	
FDC$CIRCUIT:
	LXI	H,FDC$CIRCUIT$MSG	;FDC Board only
	CALL	PRINT$STRING
	RET

IDE$CIRCUIT:
	LXI	H,IDE$CIRCUIT$MSG	;IDE Board only
	CALL	PRINT$STRING
	RET

BOTH$CIRCUITS:
	LXI	H,BOTH$CIRCUITS$MSG	;IDE + FDC Board
	CALL	PRINT$STRING
	RET

HDINIT$1:
	RET




;-----------------LOGIN TO THE IDE HARD DISK  -----------------------------------------

HDLOGIN$0:
        MVI     A,0
        OUT     IDEDrive		 ;Select Drive/CF card (A:)
        LXI	H,DPH0
        RET

HDLOGIN$1:
        MVI     A,1
        OUT     IDEDrive		;Select Drive/CF card (B:)
        LXI	H,DPH1
        RET

;------------------- SECTOR WRITE ROUTINE ----------------------------
	
					; ROUTINE WRITES 1 SECTOR TO THE DISK:
;
HDWRT:	XRA	A
	STA	ERFLG	     		;CLEAR THE ERROR FLAG

        LDA     @ADRV			;Current CPM  Drive #, (IDE Board expects a 0H or 1H)
        OUT     IDEDrive		;Select Drive/CF card (A: or B:)

	CALL	SET$LBA			;Send to drive the sector we want to read. Converting
					;CPM TRK/SEC info to Drive TRK/SEC/Head

	CALL	IDEwaitnotbusy		;make sure drive is ready
	JC	SetErrorFlag

	MVI	D,COMMANDwrite
	MVI	E,REGcommand
	CALL	IDEwr8D			;Tell drive to write a sector
	CALL	IDEwaitdrq		;wait unit it wants the data
	JC	SetErrorFlag		;If problem abort

	LHLD    @DMA			;DMA address

	IF	BANKED
	JMP	ADJBNNN

	CSEG
;=======================================

ADJBNNN:LDA     @CBNK
        PUSH    PSW
        LDA     @DBNK
        CALL    ?BNKSL
	ENDIF

	MVI	A,WRITEcfg8255
	OUT	IDEportCtrl

	MVI	B,0			;256X2 bytes
WRSEC1:	MOV	A,M
	INX	H
	OUT	IDEportA		;LOW byte on A first
	MOV	A,M
	INX	H
	OUT	IDEportB		;then HIGH byte on B
	MVI	A,REGdata
	PUSH	PSW
	OUT	IDEportC		;Send write command
	ORI	IDEwrline		;Send WR pulse
	OUT	IDEportC
	POP	PSW
	OUT	IDEportC
	DJNZ	WRSEC1

	MVI	A,READcfg8255		;Set 8255 back to read mode
	OUT	IDEportCtrl

	IF	BANKED
	POP	PSW
	CALL	?BNKSL
	JMP	CHECK$RW
;=======================================
        DSEG
	ENDIF

CHECK$RW:
	MVI	E,REGstatus		;Check R/W status when done
	CALL	IDErd8D
	MOV	A,D
	ANI	01H
	STA	ERFLG			;Ret Z if All OK
	RZ

SetErrorFlag:				;For now just return with error flag set
	XRA	A
	DCR	A
	STA	ERFLG			;Ret NZ if problem
	RET
;
;
;------------------- SECTOR READ ROUTINE ----------------------------
	
					; ROUTINE READS 1 PHYSICAL SECTOR FROM THE DRIVE:
;
HDRD:	XRA	A
	STA	ERFLG	     		;CLEAR THE ERROR FLAG

        LDA     @ADRV			;Current CPM  Drive #, (IDE Board expects a 0H or 1H)
        OUT     IDEDrive		;Select Drive/CF card (A: or B:)

	CALL	SET$LBA			;Send to drive the sector we want to read. Converting
					;CPM TRK/SEC info to Drive TRK/SEC/Head. Send before error check so info is updated
	
	CALL	IDEwaitnotbusy		;make sure drive is ready
	JC	SetErrorFlag		;Returned with NZ set if error

	MVI	D,COMMANDread
	MVI	E,REGcommand
	CALL	IDEwr8D			;Send sec read command to drive.
	CALL	IDEwaitdrq		;Wait until it's got the data
	JC	SetErrorFlag		;If problem abort
		
	LHLD  	@DMA			;DMA address

	IF	BANKED
        JMP     ADJBNKS

        CSEG				;MUST HAVE THE FOLLOWING CODE IN COMMON
;======================================================================

ADJBNKS:LDA     @CBNK
        PUSH    PSW
        LDA     @DBNK			;MUST HAVE THIS CODE IN COMMON
        CALL    ?BNKSL			;NOW DMA ADDRESS IS AT THE CORRECT BANK
	ENDIF

	MVI	B,0			;256X2 = 512

MoreRD16:
	MVI	A,REGdata		;REG regsiter address
	OUT	IDEportC	

	ORI	IDErdline		;08H+40H, Pulse RD line
	OUT	IDEportC	

	IN	IDEportA		;READ  the LOW byte first
	MOV	M,A
	INX	H
	IN	IDEportB		;THEN the HIGH byte
	MOV	M,A
	INX	H
	
	MVI	A,REGdata		;Deassert RD line
	OUT	IDEportC

	DJNZ	MoreRD16

	IF	BANKED
	POP	PSW
	CALL	?BNKSL
	ENDIF

	JMP	CHECK$RW

	IF	BANKED
	DSEG
	ENDIF
;
;					;----- SUPPORT ROUTINES --------------
;
SET$LBA:				;Note this is for the 61 sectors/track format
	LDA	@SECT			;Get CPM requested sector
	INR	A			;Disk sectors 1...MAXSEC
	MOV	D,A
	MVI	E,REGsector		;Send info to drive
	CALL	IDEwr8D

	LHLD	@TRK			;We have are in LBA mode so send directly
	MOV	D,L			;Send Low TRK#
	MVI	E,REGcylinderLSB
	CALL	IDEwr8D

	MOV	D,H			;Send High TRK#
	MVI	E,REGcylinderMSB
	CALL	IDEwr8D

	MVI	D,1			;For now, one sector at a time
	MVI	E,REGseccnt
	CALL	IDEwr8D
	RET


IDEwaitnotbusy:				;Drive READY if 01000000
	MVI	B,0FFH
	MVI	C,0FFH			;Delay, must be above 80H for 4MHz Z80. Leave longer for slower drives
MoreWait:
	MVI	E,REGstatus		;wait for RDY bit to be set
	CALL	IDErd8D
	MOV	A,D
	ANI	11000000B
	XRI	01000000B
	JZ	DoneNotbusy	
	DJNZ	MoreWait
	DCR	C
	JNZ	MoreWait
	STC				;Set carry to indicqate an error
	ret
DoneNotBusy:
	ORA	A			;Clear carry it indicate no error
	RET
	
					;Wait for the drive to be ready to transfer data.
					;Returns the drive's status in Acc
IDEwaitdrq:
	MVI	B,0FFH
	MVI	C,0FFH			;Delay, must be above 80H for 4MHz Z80. Leave longer for slower drives
MoreDRQ:
	MVI	E,REGstatus		;wait for DRQ bit to be set
	CALL	IDErd8D
	MOV	A,D
	ANI	10001000B
	CPI	00001000B
	JZ	DoneDRQ
	DJNZ	MoreDRQ
	DCR	C
	JNZ	MoreDRQ
	STC				;Set carry to indicate error
	RET
DoneDRQ:
	ORA	A			;Clear carry
	RET



;------------------------------------------------------------------
; Low Level 8 bit R/W to the drive controller.  These are the routines that talk
; directly to the drive controller registers, via the 8255 chip.  
; Note the 16 bit I/O to the drive (which is only for SEC R/W) is done directly 
; in the routines HDRD & HDWRT for speed reasons.
;
IDErd8D:				;READ 8 bits from IDE register in [E], return info in [D]
	MOV	A,E
	OUT	IDEportC		;drive address onto control lines

	ORI	IDErdline		;RD pulse pin (40H)
	OUT	IDEportC		;assert read pin

	IN	IDEportA
	MOV	D,A			;return with data in [D]

	MOV	A,E			;<---Ken Robbins suggestion
	OUT	IDEportC		;deassert RD pin first

	XRA	A
	OUT	IDEportC		;Zero all port C lines
	ret
;
;
IDEwr8D:				;WRITE Data in [D] to IDE register in [E]
	MVI	A,WRITEcfg8255		;Set 8255 to write mode
	OUT	IDEportCtrl

	MOV	A,D			;Get data put it in 8255 A port
	OUT	IDEportA
	
	MOV	A,E			;select IDE register
	OUT	IDEportC

	ORI	IDEwrline		;lower WR line
	OUT	IDEportC

	MOV	A,E			;<---Ken Robbins suggestion
	OUT	IDEportC		;deassert WR pin first

	XRA	A			;Deselect all lines including WR line
	OUT	IDEportC

	MVI	A,READcfg8255		;Config 8255 chip, read mode on return
	OUT	IDEportCtrl
	RET


;-------------------------------------COMMUNICATION WITH FPGA_DC BOARD -----------------------------------------------

DATA$TO$FPGA:				;Send CMD/data to FPGA_DC Z80
	PUSH	PSW
DATA$TO$FPGA1:
	IN	STATUS$BUS$PORT	
	CPI	0FFH			;0FFH if we have the Dual IDE/CF Card 
	JZ	NO$FPGA$DC
	ANI	80H
	JRNZ	DATA$TO$FPGA1		;Wait until bit 7 is 0 (ie. any previous data has been read by FPGA_DC board Z80)
	POP	PSW
	OUT	DATA$BUS$PORT		;This will cause the FPGA_DC board to set the S100$RTS bit
DATA$TO$FPGA2:
	IN	STATUS$BUS$PORT	
	ANI	80H
	JRNZ	DATA$TO$FPGA2		;Wait until bit 7 is 0 
	RET				;Ret NZ if error
NO$FPGA$DC
	POP	PSW
	RET
	

GET$FPGA$TO$S100:			;Get data from FPGA
	IN	STATUS$BUS$PORT		;Get S100 Status Port - Bit 0, 1 = FPGA data present, 0, no data present
	ANI	01H			;01H, Bit 0
	JZ	GET$FPGA$TO$S100	;wait for DC Board to present data 
	IN	DATA$BUS$PORT		;Return Data in [A], FPGA "hardware" will lower bit 7 of S100 Bus port 80H
	RET

; -------------- For Debugging purposes ------------------------------------
;
PRINT$STRING:					;A ROUTINE TO PRINT OUT A STRING @ [DE]
	MOV	A,M				;UP TO THE FIRST '$' ---- IF IOBYTE BIT 0 is 0
	INX	H			
	ORA	A
	RZ
	MOV	C,A
	CALL	ZCO				;Note will go to USB Port -- If bit 0 of IOBYTE = 1
	JP	PRINT$STRING


; Print an 8 bit number, located in [A]
HEXOUT:	PUSH	PSW			;No registers altered
	PUSH	B
	PUSH	PSW
	RRC
	RRC
	RRC
	RRC
	CALL	ZCONVERT
	POP	PSW
	CALL	ZCONVERT
	POP	B
	POP	PSW
	RET

ZCONVERT:	
	ANI	0FH			;HEX to ASCII and print it
	ADI	90H
	DAA
	ACI	40H
	DAA
	MOV	C,A
	CALL	ZCO
	RET


ZCI:	IN	0H			;Direct console input (used only for debugging)
	ANI	02H
	JZ	ZCI
	IN	1H			;return with character in A
	RET
;		
ZCO:	IN	0H			;console output (arrive with character in C)
	ANI	04H			;Note character is in C and A on return.
	JZ	ZCO
	MOV	A,C
	OUT	1H
	RET
	
;--------------------------------------------------------------------------------


NO$FPGA$BOARD$MSG	DB	CR,LF,BELL,'The FPGA_DC Board was not detected, will abort',CR,LF,0
MODE$2$$MSG		DB	CR,LF,'The FPGA_DC Board is set to IDE Mode 0. 9/2/21',CR,LF,0

FDC$CIRCUIT$MSG		DB	'FDC circuit detected on FPGA_DC Board',CR,LF,LF,0
IDE$CIRCUIT$MSG		DB	BELL,'IDE/CF card circuit ONLY detected on FPGA_DC Board',CR,LF,LF,0
BOTH$CIRCUITS$MSG	DB	'IDE/CF card and FDC circuits detected on FPGA_DC Board',CR,LF,LF,0

ERFLG:		DB	0H
STACK$STORE	DW	0H
		DS	100H
LOCAL$STACK	DB	0FFH		;Not currently used

;END
